@@ -414,9 +414,14 @@ class GenerateFencingParametersAction(base.TripleOAction):
                 # New-style hardware types (ipmi, etc)
                 driver_proto = node['pm_type']
 
-            if driver_proto in {'ipmi', 'ipmitool', 'drac', 'idrac', 'ilo'}:
+            if driver_proto in {'ipmi', 'ipmitool', 'drac', 'idrac', 'ilo',
+                                'redfish'}:
                 # IPMI fencing driver
-                node_data["agent"] = "fence_ipmilan"
+                if driver_proto == "redfish":
+                    node_data["agent"] = "fence_redfish"
+                    params["systems_uri"] = node["pm_system_id"]
+                else:
+                    node_data["agent"] = "fence_ipmilan"
                 params["ipaddr"] = node["pm_addr"]
                 params["passwd"] = node["pm_password"]
                 params["login"] = node["pm_user"]
@@ -425,6 +430,11 @@ class GenerateFencingParametersAction(base.TripleOAction):
                         hostmap[mac_addr]["compute_name"]
                 if "pm_port" in node:
                     params["ipport"] = node["pm_port"]
+                if "redfish_verify_ca" in node:
+                    if node["redfish_verify_ca"] == "false":
+                        params["ssl_insecure"] = "true"
+                    else:
+                        params["ssl_insecure"] = "false"
                 if self.ipmi_lanplus:
                     params["lanplus"] = self.ipmi_lanplus
                 if self.delay:
@@ -433,6 +443,20 @@ class GenerateFencingParametersAction(base.TripleOAction):
                     params["cipher"] = self.ipmi_cipher
                 if self.ipmi_level:
                     params["privlvl"] = self.ipmi_level
+            elif driver_proto in {'staging-ovirt'}:
+                # fence_rhevm
+                node_data["agent"] = "fence_rhevm"
+                params["ipaddr"] = node["pm_addr"]
+                params["passwd"] = node["pm_password"]
+                params["login"] = node["pm_user"]
+                params["port"] = node["pm_vm_name"]
+                params["ssl"] = 1
+                params["ssl_insecure"] = 1
+                if hostmap:
+                    params["pcmk_host_list"] = \
+                        hostmap[mac_addr]["compute_name"]
+                if self.delay:
+                    params["delay"] = self.delay
             else:
                 error = ("Unable to generate fencing parameters for %s" %
                          node["pm_type"])
