@@ -110,8 +110,7 @@ class DeletePlanAction(base.TripleOAction):
         # heat throws HTTPNotFound if the stack is not found
         try:
             stack = self.get_orchestration_client(context).stacks.get(
-                self.container
-            )
+                self.container, resolve_outputs=False)
         except heatexceptions.HTTPNotFound:
             pass
         else:
@@ -123,6 +122,8 @@ class DeletePlanAction(base.TripleOAction):
             swiftutils.delete_container(swift, self.container)
             swiftutils.delete_container(swift,
                                         "%s-swift-rings" % self.container)
+            swiftutils.delete_container(swift,
+                                        "%s-messages" % self.container)
         except swiftexceptions.ClientException as ce:
             LOG.exception("Swift error deleting plan.")
             error_text = ce.msg
@@ -281,27 +282,19 @@ class UpdatePlanFromDirAction(base.TripleOAction):
 class UpdatePlanEnvironmentAction(base.TripleOAction):
     """Updates the plan environment values
 
-    Updates a plan environment values with the given parameters:
-        Add new parameter
-        Delete parameter
+    Updates a plan environment values - when parameter is specified
+    the value is merged with the env_key (useful for parameter_defaults)
+    otherwise the value for env_key is replaced.
 
-    :param parameter: key value of the parameter to add or delete
-    :param value: value of the parameter to add or delete
-    :param delete: True if the parameter should be deleted from the env
     :param env_key: environment key that should be one of the keys present
-                    in the plan environment dictionary:
-                         'passwords',
-                         'description',
-                         'parameter_defaults',
-                         'environments',
-                         'version',
-                         'template',
-                         'resource_registry',
-                         'name'
+                    in the plan environment dictionary
+    :param parameter: key value of the parameter add/delete/modify
+    :param value: value of the parameter or of env_key when no parameter
+    :param delete: True if the parameter should be deleted from env_key
     :param container: name of the Swift container / plan name
     """
 
-    def __init__(self, parameter, env_key, value=None, delete=False,
+    def __init__(self, env_key, parameter=None, value=None, delete=False,
                  container=constants.DEFAULT_CONTAINER_NAME):
         super(UpdatePlanEnvironmentAction, self).__init__()
         self.container = container
@@ -314,17 +307,26 @@ class UpdatePlanEnvironmentAction(base.TripleOAction):
         try:
             swift = self.get_object_client(context)
             plan_env = plan_utils.get_env(swift, self.container)
-            if self.env_key in plan_env.keys():
-                if self.delete:
-                    try:
-                        plan_env[self.env_key].pop(self.parameter)
-                    except KeyError:
-                        pass
-                else:
-                    plan_env[self.env_key].update({self.parameter: self.value})
-            else:
+
+            if self.env_key not in plan_env:
                 msg = "The environment key doesn't exist: %s" % self.env_key
                 return actions.Result(error=msg)
+
+            if self.delete:
+                if not self.parameter:
+                    raise ValueError("delete must specify a parameter")
+                plan_env[self.env_key].pop(self.parameter, None)
+            elif self.parameter:
+                plan_env[self.env_key][self.parameter] = self.value
+            elif self.value is not None:
+                plan_env[self.env_key] = self.value
+            else:
+                # Nothing to modify
+                return
+
+            plan_utils.update_in_env(swift, plan_env,
+                                     self.env_key,
+                                     value=plan_env[self.env_key])
         except swiftexceptions.ClientException as err:
             msg = "Error attempting an operation on container: %s" % err
             return actions.Result(error=msg)
