@@ -12,6 +12,7 @@
 
 import datetime
 import fixtures
+import git
 import mock
 import os
 import uuid
@@ -32,6 +33,7 @@ class TestConfig(base.TestCase):
     def setUp(self):
         super(TestConfig, self).setUp()
 
+    @patch.object(ooo_config.Config, 'initialize_git_repo')
     @patch.object(ooo_config.shutil, 'copyfile')
     @patch.object(ooo_config.Config, '_mkdir')
     @patch.object(ooo_config.Config, '_open_file')
@@ -40,7 +42,8 @@ class TestConfig(base.TestCase):
                                               mock_rmtree,
                                               mock_open,
                                               mock_mkdir,
-                                              mock_copyfile):
+                                              mock_copyfile,
+                                              mock_git_init):
         config_type_list = ['config_settings', 'global_config_settings',
                             'logging_sources', 'monitoring_subscriptions',
                             'service_config_settings',
@@ -56,6 +59,7 @@ class TestConfig(base.TestCase):
         self.config = ooo_config.Config(heat)
         self.config.download_config('overcloud', '/tmp/tht', config_type_list)
 
+        mock_git_init.assert_called_once_with('/tmp/tht')
         expected_mkdir_calls = [call('/tmp/tht/%s' % r) for r in fake_role]
         mock_mkdir.assert_has_calls(expected_mkdir_calls, any_order=True)
         expected_calls = []
@@ -71,6 +75,7 @@ class TestConfig(base.TestCase):
                                             (role, config))]
         mock_open.assert_has_calls(expected_calls, any_order=True)
 
+    @patch.object(ooo_config.Config, 'initialize_git_repo')
     @patch.object(ooo_config.shutil, 'copyfile')
     @patch.object(ooo_config.Config, '_mkdir')
     @patch.object(ooo_config.Config, '_open_file')
@@ -79,7 +84,8 @@ class TestConfig(base.TestCase):
                                               mock_rmtree,
                                               mock_open,
                                               mock_mkdir,
-                                              mock_copyfile):
+                                              mock_copyfile,
+                                              mock_git_init):
 
         expected_config_type = 'config_settings'
         fake_role = [role for role in
@@ -96,6 +102,7 @@ class TestConfig(base.TestCase):
                           for r in fake_role]
         mock_mkdir.assert_has_calls(expected_mkdir_calls, any_order=True)
         mock_open.assert_has_calls(expected_calls, any_order=True)
+        mock_git_init.assert_called_once_with('/tmp/tht')
 
     @patch.object(ooo_config.git, 'Repo')
     @mock.patch('os.mkdir')
@@ -224,7 +231,12 @@ class TestConfig(base.TestCase):
 
         return deployment_data, configs
 
-    def _get_config_dict(self, deployment):
+    def _get_deployment_id(self, deployment):
+        return deployment.attributes['value']['deployment']
+
+    def _get_config_dict(self, deployment_id):
+        deployment = list(filter(
+            lambda d: d.id == deployment_id, self.deployments))[0]
         config = self.configs[deployment.attributes['value']['config']].copy()
         config['inputs'] = []
         config['inputs'].append(dict(
@@ -239,14 +251,17 @@ class TestConfig(base.TestCase):
             file_name)
         return yaml.safe_load(open(file_path).read())
 
+    @patch.object(ooo_config.Config, 'initialize_git_repo')
+    @patch('tripleo_common.utils.config.Config.get_deployment_resource_id')
     @patch('tripleo_common.utils.config.Config.get_config_dict')
     @patch('tripleo_common.utils.config.Config.get_deployment_data')
-    def test_config_download(self, mock_deployment_data, mock_config_dict):
+    def test_config_download(self, mock_deployment_data, mock_config_dict,
+                             mock_deployment_resource_id,
+                             mock_git_init):
         heat = mock.MagicMock()
         self.config = ooo_config.Config(heat)
         stack = mock.MagicMock()
         heat.stacks.get.return_value = stack
-
         stack.outputs = [
             {'output_key': 'RoleNetHostnameMap',
              'output_value': {
@@ -266,17 +281,28 @@ class TestConfig(base.TestCase):
                      'Compute': [
                          'a7db3010-a51f-4ae0-a791-2364d629d20d',
                          '8b07cd31-3083-4b88-a433-955f72039e2c',
-                         '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}}]
+                         '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}},
+            {'output_key': 'RoleGroupVars',
+             'output_value': {
+                 'Controller': {
+                     'any_errors_fatal': 'yes',
+                     'max_fail_percentage': 15},
+                 'Compute': {
+                     'any_errors_fatal': 'yes',
+                     'max_fail_percentage': 15},
+                 }}]
         deployment_data, configs = \
             self._get_config_data('config_data.yaml')
         self.configs = configs
-
+        self.deployments = deployment_data
         mock_deployment_data.return_value = deployment_data
+        mock_deployment_resource_id.side_effect = self._get_deployment_id
         mock_config_dict.side_effect = self._get_config_dict
 
         self.tmp_dir = self.useFixture(fixtures.TempDir()).path
         tmp_path = self.config.download_config(stack, self.tmp_dir)
 
+        mock_git_init.assert_called_once_with(self.tmp_dir)
         for f in ['Controller',
                   'Compute', ]:
 
@@ -335,15 +361,18 @@ class TestConfig(base.TestCase):
                                     'overcloud-novacompute-2',
                                     d)))
 
+    @patch.object(ooo_config.Config, 'initialize_git_repo')
+    @patch('tripleo_common.utils.config.Config.get_deployment_resource_id')
     @patch('tripleo_common.utils.config.Config.get_config_dict')
     @patch('tripleo_common.utils.config.Config.get_deployment_data')
     def test_config_download_os_apply_config(
-        self, mock_deployment_data, mock_config_dict):
+        self, mock_deployment_data, mock_config_dict,
+        mock_deployment_resource_id, mock_git_init):
         heat = mock.MagicMock()
         self.config = ooo_config.Config(heat)
         stack = mock.MagicMock()
         heat.stacks.get.return_value = stack
-
+        heat.resources.get.return_value = mock.MagicMock()
         stack.outputs = [
             {'output_key': 'RoleNetHostnameMap',
              'output_value': {
@@ -363,7 +392,16 @@ class TestConfig(base.TestCase):
                      'Compute': [
                          'a7db3010-a51f-4ae0-a791-2364d629d20d',
                          '8b07cd31-3083-4b88-a433-955f72039e2c',
-                         '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}}]
+                         '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}},
+            {'output_key': 'RoleGroupVars',
+             'output_value': {
+                 'Controller': {
+                     'any_errors_fatal': 'yes',
+                     'max_fail_percentage': 15},
+                 'Compute': {
+                     'any_errors_fatal': 'yes',
+                     'max_fail_percentage': 15},
+             }}]
         deployment_data, configs = \
             self._get_config_data('config_data.yaml')
 
@@ -386,36 +424,112 @@ class TestConfig(base.TestCase):
         deployment_data.append(deployment_mock)
 
         self.configs = configs
-
+        self.deployments = deployment_data
         mock_deployment_data.return_value = deployment_data
         mock_config_dict.side_effect = self._get_config_dict
+        mock_deployment_resource_id.side_effect = self._get_deployment_id
 
         self.tmp_dir = self.useFixture(fixtures.TempDir()).path
         with warnings.catch_warnings(record=True) as w:
             self.config.download_config(stack, self.tmp_dir)
+            mock_git_init.assert_called_once_with(self.tmp_dir)
             self.assertEqual(1, len(w))
             assert issubclass(w[-1].category, DeprecationWarning)
             assert "group:os-apply-config is deprecated" in str(w[-1].message)
 
+    @patch.object(ooo_config.Config, 'initialize_git_repo')
+    @patch('tripleo_common.utils.config.Config.get_deployment_resource_id')
     @patch('tripleo_common.utils.config.Config.get_deployment_data')
-    def test_config_download_no_deployment_name(self, mock_deployment_data):
+    def test_config_download_no_deployment_name(
+        self, mock_deployment_data, mock_deployment_resource_id,
+        mock_git_init):
         heat = mock.MagicMock()
         self.config = ooo_config.Config(heat)
         stack = mock.MagicMock()
         heat.stacks.get.return_value = stack
+        heat.resources.get.return_value = mock.MagicMock()
 
         deployment_data, _ = self._get_config_data('config_data.yaml')
 
         # Delete the name of the first deployment and his parent.
         del deployment_data[0].attributes['value']['name']
         deployment_data[0].parent_resource = None
+        self.deployments = deployment_data
 
         mock_deployment_data.return_value = deployment_data
+        mock_deployment_resource_id.side_effect = self._get_deployment_id
 
         self.tmp_dir = self.useFixture(fixtures.TempDir()).path
         self.assertRaises(ValueError,
                           self.config.download_config, stack, self.tmp_dir)
+        mock_git_init.assert_called_once_with(self.tmp_dir)
 
+    @patch.object(ooo_config.Config, 'initialize_git_repo')
+    @patch('tripleo_common.utils.config.Config.get_deployment_resource_id')
+    @patch('tripleo_common.utils.config.Config.get_config_dict')
+    @patch('tripleo_common.utils.config.Config.get_deployment_data')
+    def test_config_download_no_deployment_uuid(self, mock_deployment_data,
+                                                mock_config_dict,
+                                                mock_deployment_resource_id,
+                                                mock_git_init):
+        heat = mock.MagicMock()
+        self.config = ooo_config.Config(heat)
+        stack = mock.MagicMock()
+        heat.stacks.get.return_value = stack
+        heat.resources.get.return_value = mock.MagicMock()
+
+        stack.outputs = [
+            {'output_key': 'RoleNetHostnameMap',
+             'output_value': {
+                 'Controller': {
+                     'ctlplane': [
+                         'overcloud-controller-0.ctlplane.localdomain']},
+                 'Compute': {
+                     'ctlplane': [
+                         'overcloud-novacompute-0.ctlplane.localdomain',
+                         'overcloud-novacompute-1.ctlplane.localdomain',
+                         'overcloud-novacompute-2.ctlplane.localdomain']}}},
+            {'output_key': 'ServerIdData',
+             'output_value': {
+                 'server_ids': {
+                     'Controller': [
+                         '00b3a5e1-5e8e-4b55-878b-2fa2271f15ad'],
+                     'Compute': [
+                         'a7db3010-a51f-4ae0-a791-2364d629d20d',
+                         '8b07cd31-3083-4b88-a433-955f72039e2c',
+                         '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}},
+            {'output_key': 'RoleGroupVars',
+             'output_value': {
+                 'Controller': {
+                     'any_errors_fatal': 'yes',
+                     'max_fail_percentage': 15},
+                 'Compute': {
+                     'any_errors_fatal': 'yes',
+                     'max_fail_percentage': 15},
+             }}]
+        deployment_data, configs = self._get_config_data('config_data.yaml')
+
+        # Set the deployment to TripleOSoftwareDeployment for the first
+        # deployment
+        deployment_data[0].attributes['value']['deployment'] = \
+            'TripleOSoftwareDeployment'
+
+        # Set the physical_resource_id as '' for the second deployment
+        deployment_data[1].attributes['value']['deployment'] = ''
+
+        self.configs = configs
+        self.deployments = deployment_data
+        mock_deployment_data.return_value = deployment_data
+        mock_config_dict.side_effect = self._get_config_dict
+        mock_deployment_resource_id.side_effect = self._get_deployment_id
+
+        self.tmp_dir = self.useFixture(fixtures.TempDir()).path
+        with warnings.catch_warnings(record=True) as w:
+            self.config.download_config(stack, self.tmp_dir)
+            assert "Skipping deployment" in str(w[-1].message)
+            assert "Skipping deployment" in str(w[-2].message)
+
+    @patch.object(ooo_config.Config, 'initialize_git_repo')
     @patch.object(ooo_config.git, 'Repo')
     @patch.object(ooo_config.shutil, 'copyfile')
     @patch.object(ooo_config.Config, '_mkdir')
@@ -428,7 +542,8 @@ class TestConfig(base.TestCase):
                                                    mock_open,
                                                    mock_mkdir,
                                                    mock_copyfile,
-                                                   mock_repo):
+                                                   mock_repo,
+                                                   mock_git_init):
         config_type_list = ['config_settings', 'global_config_settings',
                             'logging_sources', 'monitoring_subscriptions',
                             'service_config_settings',
@@ -446,6 +561,7 @@ class TestConfig(base.TestCase):
         self.config.download_config('overcloud', '/tmp/tht', config_type_list,
                                     False)
 
+        mock_git_init.assert_called_once_with('/tmp/tht')
         expected_rmtree_calls = [call('/tmp/tht')]
         mock_rmtree.assert_has_calls(expected_rmtree_calls)
 
@@ -475,6 +591,14 @@ class TestConfig(base.TestCase):
         expected_rmtree_calls = [call('/tmp/tht')]
         mock_rmtree.assert_has_calls(expected_rmtree_calls)
 
+    def test_initialize_git_repo(self):
+        heat = mock.MagicMock()
+        heat.stacks.get.return_value = fakes.create_tht_stack()
+        self.config = ooo_config.Config(heat)
+        self.tmp_dir = self.useFixture(fixtures.TempDir()).path
+        repo = self.config.initialize_git_repo(self.tmp_dir)
+        self.assertIsInstance(repo, git.Repo)
+
     @patch('tripleo_common.utils.config.Config.get_config_dict')
     @patch('tripleo_common.utils.config.Config.get_deployment_data')
     def test_write_config(self, mock_deployment_data, mock_config_dict):
@@ -502,15 +626,24 @@ class TestConfig(base.TestCase):
                      'Compute': [
                          'a7db3010-a51f-4ae0-a791-2364d629d20d',
                          '8b07cd31-3083-4b88-a433-955f72039e2c',
-                         '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}}]
+                         '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}},
+            {'output_key': 'RoleGroupVars',
+             'output_value': {
+                 'Controller': {
+                     'any_errors_fatal': 'yes',
+                     'max_fail_percentage': 15},
+                 'Compute': {
+                     'any_errors_fatal': 'yes',
+                     'max_fail_percentage': 15},
+             }}]
         deployment_data, configs = \
             self._get_config_data('config_data.yaml')
         self.configs = configs
+        self.deployments = deployment_data
 
         stack_data = self.config.fetch_config('overcloud')
         mock_deployment_data.return_value = deployment_data
         mock_config_dict.side_effect = self._get_config_dict
-
         config_dir = self.useFixture(fixtures.TempDir()).path
 
         self.config.write_config(stack_data, 'overcloud', config_dir)
