@@ -13,6 +13,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 import base64
+import hashlib
+import hmac
 import logging
 import os
 import paramiko
@@ -31,7 +33,8 @@ KEYSTONE_FERNET_REPO = '/etc/keystone/fernet-keys/'
 LOG = logging.getLogger(__name__)
 
 
-def generate_passwords(mistralclient=None, stack_env=None):
+def generate_passwords(mistralclient=None, stack_env=None,
+                       rotate_passwords=False):
     """Create the passwords needed for deploying OpenStack via t-h-t.
 
     This will create the set of passwords required by the undercloud and
@@ -46,7 +49,8 @@ def generate_passwords(mistralclient=None, stack_env=None):
     for name in constants.PASSWORD_PARAMETER_NAMES:
         # Support users upgrading from Mitaka or otherwise creating a plan for
         # a Heat stack that already exists.
-        if stack_env and name in stack_env.get('parameter_defaults', {}):
+        if (stack_env and name in stack_env.get('parameter_defaults', {}) and
+                not rotate_passwords):
             passwords[name] = stack_env['parameter_defaults'][name]
         elif name.startswith("Ceph"):
             if name == "CephClusterFSID":
@@ -85,6 +89,10 @@ def generate_passwords(mistralclient=None, stack_env=None):
             passwords[name] = passlib.pwd.genword(length=10)
         elif name.startswith("HeatAuthEncryptionKey"):
             passwords[name] = passlib.pwd.genword(length=32)
+        elif name.startswith("OctaviaServerCertsKeyPassphrase"):
+            passwords[name] = passlib.pwd.genword(length=32)
+        elif name.startswith("DesignateRndcKey"):
+            passwords[name] = create_rndc_key_secret()
         else:
             passwords[name] = passlib.pwd.genword(length=_MIN_PASSWORD_SIZE)
     return passwords
@@ -133,3 +141,12 @@ def create_ssh_keypair(comment=None, bits=2048):
         'private_key': private_key,
         'public_key': public_key,
     }
+
+
+def create_rndc_key_secret():
+    # The rndc key secret is a base64-encoded hmac-sha256 value
+    h = hmac.new(
+        passlib.pwd.genword(length=_MIN_PASSWORD_SIZE).encode('utf-8'),
+        msg=passlib.pwd.genword(length=_MIN_PASSWORD_SIZE).encode('utf-8'),
+        digestmod=hashlib.sha256)
+    return base64.b64encode(h.digest())
